<?php

    /** @noinspection PhpMissingFieldTypeInspection */

    namespace FederationCLI;

    use FederationLib\FederationLib;

    class InteractiveMode
    {
        /**
         * The current menu the user is in
         *
         * @var string
         */
        private static $current_menu = 'Main';

        /**
         * An array of menu pointers to functions
         *
         * @var string[]
         */
        private static $menu_pointers = [
            'ClientManager' => 'FederationCLI\InteractiveMode\ClientManager::processCommand',
            'ConfigManager' => 'FederationCLI\InteractiveMode\ConfigurationManager::processCommand'
        ];

        private static $help_pointers =[
            'ClientManager' => 'FederationCLI\InteractiveMode\ClientManager::help',
            'ConfigManager' => 'FederationCLI\InteractiveMode\ConfigurationManager::help'
        ];

        /**
         * @var FederationLib|null
         */
        private static $federation_lib = null;

        /**
         * Main entry point for the interactive mode
         *
         * @param array $args
         * @return void
         */
        public static function main(array $args=[]): void
        {
            while(true)
            {
                print(sprintf('federation@%s:~$ ', self::$current_menu));
                $input = trim(fgets(STDIN));

                self::processCommand($input);
            }
        }

        /**
         * Processes a command from the user
         *
         * @param string $input
         * @return void
         */
        private static function processCommand(string $input): void
        {
            $parsed_input = Utilities::parseShellInput($input);

            switch(strtolower($parsed_input['command']))
            {
                case 'help':
                    print('Available commands:' . PHP_EOL);
                    print('  help - displays the help menu for the current menu and global commands' . PHP_EOL);
                    print('  client_manager - enter client manager mode' . PHP_EOL);
                    print('  config_manager - enter config manager mode' . PHP_EOL);
                    print('  clear - clears the screen' . PHP_EOL);
                    print('  exit - exits the current menu, if on the main menu, exits the program' . PHP_EOL);

                    if(isset(self::$help_pointers[self::$current_menu]))
                    {
                        call_user_func(self::$help_pointers[self::$current_menu]);
                    }

                    break;

                case 'client_manager':
                    self::$current_menu = 'ClientManager';
                    break;

                case 'config_manager':
                    self::$current_menu = 'ConfigManager';
                    break;

                case 'clear':
                    print(chr(27) . "[2J" . chr(27) . "[;H");
                    break;

                case 'exit':
                    if(self::$current_menu != 'Main')
                    {
                        self::$current_menu = 'Main';
                        break;
                    }

                    exit(0);

                default:
                    if(!isset(self::$menu_pointers[self::$current_menu]))
                    {
                        print(sprintf('Unknown command: %s', $parsed_input['command']) . PHP_EOL);
                        break;
                    }

                    call_user_func(self::$menu_pointers[self::$current_menu], $input);
                    break;
            }
        }

        /**
         * Returns the current menu
         *
         * @return string
         */
        public static function getCurrentMenu(): string
        {
            return self::$current_menu;
        }

        /**
         * Sets the current menu to the specified value
         *
         * @param string $current_menu
         */
        public static function setCurrentMenu(string $current_menu): void
        {
            self::$current_menu = $current_menu;
        }

        /**
         * Returns the FederationLib instance
         *
         * @return FederationLib
         */
        public static function getFederationLib(): FederationLib
        {
            if(self::$federation_lib == null)
            {
                self::$federation_lib = new FederationLib();
            }

            return self::$federation_lib;
        }
    }