<?php

    namespace FederationCLI\InteractiveMode;

    use Exception;
    use FederationCLI\Utilities;
    use FederationLib\Classes\Configuration;

    class ConfigurationManager
    {
        /**
         * @param string $input
         * @return void
         */
        public static function processCommand(string $input): void
        {
            $parsed_input = Utilities::parseShellInput($input);

            switch(strtolower($parsed_input['command']))
            {
                case 'read':
                    self::read($parsed_input['args'][0] ?? null);
                    break;

                case 'write':
                    self::write($parsed_input['args'][0] ?? null, $parsed_input['args'][1] ?? null);
                    break;

                case 'save':
                    self::save();
                    break;

                default:
                    print(sprintf('Unknown command: %s', $parsed_input['command']) . PHP_EOL);
                    break;
            }
        }

        /**
         * Displays the help message for the client manager
         *
         * @return void
         */
        public static function help(): void
        {
            print('Configuration manager commands:' . PHP_EOL);
            print('  read - reads the current configuration' . PHP_EOL);
            print('  read <key> - reads the value of the specified configuration key' . PHP_EOL);
            print('  write <key> <value> - writes the value of the specified configuration key' . PHP_EOL);
            print('  save - saves the current configuration to disk' . PHP_EOL);
        }

        /**
         * Reads the current configuration or the value of a specific key
         *
         * @param string|null $key
         * @return void
         */
        private static function read(?string $key=null): void
        {
            if($key === null)
            {
                $value = Configuration::getConfiguration();
            }
            else
            {
                $value = Configuration::getConfigurationObject()->get($key);
            }

            if(is_null($value))
            {
                print('No value found for key: ' . $key . PHP_EOL);
            }
            elseif(is_array($value))
            {
                print(json_encode($value, JSON_PRETTY_PRINT) . PHP_EOL);
            }
            else
            {
                print($value . PHP_EOL);
            }
        }

        /**
         * Writes the value of a specific configuration key
         *
         * @param string $key
         * @param string $value
         * @return void
         */
        private static function write(string $key, string $value): void
        {
            Configuration::getConfigurationObject()->set($key, $value);
        }

        /**
         * Writes the current configuration to disk
         *
         * @return void
         */
        private static function save(): void
        {
            try
            {
                Configuration::getConfigurationObject()->save();
            }
            catch(Exception $e)
            {
                print('Failed to save configuration: ' . $e->getMessage() . PHP_EOL);
                Utilities::printExceptionStack($e);
            }
        }
    }