<?php

    /** @noinspection PhpMissingFieldTypeInspection */

    namespace FederationLib\Classes;

    use Exception;
    use FederationLib\Classes\Configuration\CacheServerConfiguration;
    use RuntimeException;

    class Configuration
    {
        /**
         * @var \ConfigLib\Configuration|null
         */
        private static $configuration;

        /**
         * Returns the full raw configuration array.
         *
         * @return array
         */
        public static function getConfiguration(): array
        {
            if(self::$configuration === null)
            {
                self::$configuration = new \ConfigLib\Configuration('federation');

                /** Database Configuration */
                self::$configuration->setDefault('database.driver', 'mysqli');
                self::$configuration->setDefault('database.host', 'localhost');
                self::$configuration->setDefault('database.port', 3306);
                self::$configuration->setDefault('database.name', 'federation');
                self::$configuration->setDefault('database.username', 'root');
                self::$configuration->setDefault('database.password', 'root');
                self::$configuration->setDefault('database.charset', 'utf8mb4');
                self::$configuration->setDefault('database.reconnect_interval', 1800);

                /** Multi-Cache Configuration */
                self::$configuration->setDefault('cache_system.enabled', true);
                // Cache System Configuration
                // Client Objects
                self::$configuration->setDefault('cache_system.cache.client_objects_enabled', true);
                self::$configuration->setDefault('cache_system.cache.client_objects_ttl', 200);
                self::$configuration->setDefault('cache_system.cache.client_objects_server_preference', 'redis_master');
                self::$configuration->setDefault('cache_system.cache.client_objects_server_fallback', 'any');
                // Peer Objects
                self::$configuration->setDefault('cache_system.cache.peer_objects_enabled', true);
                self::$configuration->setDefault('cache_system.cache.peer_objects_ttl', 200);
                self::$configuration->setDefault('cache_system.cache.peer_objects_server_preference', 'redis_master');
                self::$configuration->setDefault('cache_system.cache.peer_objects_server_fallback', 'any');
                // Redis Configuration
                self::$configuration->setDefault('cache_system.servers.redis_master.enabled', true);
                self::$configuration->setDefault('cache_system.servers.redis_master.host', 'localhost');
                self::$configuration->setDefault('cache_system.servers.redis_master.port', 6379);
                self::$configuration->setDefault('cache_system.servers.redis_master.driver', 'redis');
                self::$configuration->setDefault('cache_system.servers.redis_master.priority', 1);
                self::$configuration->setDefault('cache_system.servers.redis_master.username', null);
                self::$configuration->setDefault('cache_system.servers.redis_master.password', null);
                self::$configuration->setDefault('cache_system.servers.redis_master.database', null);
                self::$configuration->setDefault('cache_system.servers.redis_master.reconnect_interval', 1800);
                // Memcached Configuration
                self::$configuration->setDefault('cache_system.servers.memcached_master.enabled', false);
                self::$configuration->setDefault('cache_system.servers.memcached_master.host', 'localhost');
                self::$configuration->setDefault('cache_system.servers.memcached_master.port', 11211);
                self::$configuration->setDefault('cache_system.servers.memcached_master.driver', 'memcached');
                self::$configuration->setDefault('cache_system.servers.memcached_master.priority', 1);
                self::$configuration->setDefault('cache_system.servers.memcached_master.username', null);
                self::$configuration->setDefault('cache_system.servers.memcached_master.password', null);
                self::$configuration->setDefault('cache_system.servers.memcached_master.database', null);
                self::$configuration->setDefault('cache_system.servers.memcached_master.reconnect_interval', 1800);



                /** Federation Configuration */
                self::$configuration->setDefault('federation.events_retention', 1209600); // Two Weeks
                self::$configuration->setDefault('federation.events_enabled.generic', true);
                self::$configuration->setDefault('federation.events_enabled.client_Created', true);


                /** Save the configuration's default values if they don't exist */
                try
                {
                    self::$configuration->save();
                }
                catch(Exception $e)
                {
                    throw new RuntimeException('Failed to save configuration: ' . $e->getMessage(), $e);
                }
            }

            return self::$configuration->getConfiguration();
        }

        /**
         * Returns the configuration object.
         *
         * @return \ConfigLib\Configuration
         */
        public static function getConfigurationObject(): \ConfigLib\Configuration
        {
            if(self::$configuration === null)
            {
                self::getConfiguration();
            }

            return self::$configuration;
        }

        /**
         * Returns driver of the database.
         *
         * @return string
         */
        public static function getDatabaseDriver(): string
        {
            return self::getConfiguration()['database']['driver'];
        }

        /**
         * Returns the host of the database.
         *
         * @return string
         */
        public static function getDatabaseHost(): string
        {
            return self::getConfiguration()['database']['host'];
        }

        /**
         * Returns the port of the database.
         *
         * @return int
         */
        public static function getDatabasePort(): int
        {
            return (int)self::getConfiguration()['database']['port'];
        }

        /**
         * Returns the name of the database.
         *
         * @return string
         */
        public static function getDatabaseName(): string
        {
            return self::getConfiguration()['database']['name'];
        }

        /**
         * Returns the username of the database.
         *
         * @return string
         */
        public static function getDatabaseUsername(): string
        {
            return self::getConfiguration()['database']['username'];
        }

        /**
         * Returns the password of the database.
         *
         * @return string|null
         */
        public static function getDatabasePassword(): ?string
        {
            $password = self::getConfiguration()['database']['password'];

            if($password === '')
            {
                return null;
            }

            return $password;
        }

        /**
         * Returns the charset of the database.
         *
         * @return string
         */
        public static function getDatabaseCharset(): string
        {
            return self::getConfiguration()['database']['charset'];
        }

        /**
         * Returns the interval in seconds to reconnect to the database.
         *
         * @return int
         */
        public static function getDatabaseReconnectInterval(): int
        {
            return (int)self::getConfiguration()['database']['reconnect_interval'];
        }

        /**
         * Returns True if the cache system is enabled for FederationLib
         * and False if not, based on the configuration.
         *
         * @return bool
         */
        public static function isCacheSystemEnabled(): bool
        {
            return (bool)self::getConfiguration()['cache_system']['enabled'];
        }

        /**
         * Returns the configuration for all the cache servers
         *
         * @return CacheServerConfiguration[]
         */
        public static function getCacheServers(): array
        {
            $results = [];
            foreach(self::getConfiguration()['cache_system']['servers'] as $server_name => $server)
            {
                if($server['enabled'] === true)
                {
                    $results[] = new CacheServerConfiguration($server_name, $server);
                }
            }

            return $results;
        }
    }