<?php

    /** @noinspection PhpMissingFieldTypeInspection */

    namespace FederationLib\Classes;

    use Exception;
    use LogLib\Log;
    use RedisException;

    class Redis
    {
        /**
         * @var int|null
         */
        private static $redis_last_connection_time;

        /**
         * @var \Redis|null
         */
        private static $redis_connection;

        /**
         * Returns/Establishes a connection to the redis server.
         * Returns null if redis is disabled.
         *
         * @return \Redis|null
         * @throws RedisException
         */
        public static function getConnection(): ?\Redis
        {
            if(!Configuration::isRedisEnabled())
            {
                return null;
            }

            if(self::$redis_connection === null)
            {
                try
                {
                    Log::info('net.nosial.federationlib', sprintf('Connecting to the redis server: %s:%s', Configuration::getRedisHost(), Configuration::getRedisPort()));
                    $redis = new \Redis();
                    $redis->connect(Configuration::getRedisHost(), Configuration::getRedisPort());
                    if(Configuration::getRedisPassword() !== null)
                    {
                        $redis->auth(Configuration::getRedisPassword());
                    }
                    $redis->select(Configuration::getRedisDatabase());
                }
                catch(Exception $e)
                {
                    throw new RedisException('Failed to connect to the redis server: ' . $e->getMessage(), $e->getCode(), $e);
                }

                self::$redis_connection = $redis;
                self::$redis_last_connection_time = time();
            }
            else
            {
                if(self::$redis_last_connection_time === null || self::$redis_last_connection_time < (time() - Configuration::getRedisReconnectInterval()))
                {
                    Log::info('net.nosial.federationlib', 'Interval to reconnect to the redis server has been reached, reconnecting...');

                    try
                    {
                        self::$redis_connection->close();
                    }
                    catch(Exception $e)
                    {
                        // Do nothing
                        unset($e);
                    }

                    self::$redis_connection = null;
                    return self::getConnection();
                }
            }

            return self::$redis_connection;
        }

        /**
         * Returns the last time the redis server was connected to.
         *
         * @return int|null
         */
        public static function getRedisLastConnectionTime(): ?int
        {
            return self::$redis_last_connection_time;
        }
    }