<?php

    namespace FederationLib\Managers;

    use Doctrine\DBAL\ParameterType;
    use Exception;
    use FederationLib\Classes\Configuration;
    use FederationLib\Classes\Database;
    use FederationLib\Enums\DatabaseTables;
    use FederationLib\Enums\Misc;
    use FederationLib\Enums\Standard\InternetPeerType;
    use FederationLib\Enums\Standard\PeerType;
    use FederationLib\Enums\Standard\UserPeerType;
    use FederationLib\Exceptions\DatabaseException;
    use FederationLib\Exceptions\Standard\PeerNotFoundException;
    use FederationLib\Exceptions\Standard\UnsupportedPeerType;
    use FederationLib\FederationLib;
    use FederationLib\Objects\Client;
    use FederationLib\Objects\ParsedFederatedAddress;
    use FederationLib\Objects\Peer;
    use LogLib\Log;

    class PeerManager
    {
        /**
         * @var FederationLib
         */
        private $federationLib;

        /**
         * @param FederationLib $federationLib
         */
        public function __construct(FederationLib $federationLib)
        {
            $this->federationLib = $federationLib;
        }

        /**
         * Returns the Peer Type of the federated address, returns "unknown" if the
         * type is not supported by the server
         *
         * @param string $type
         * @return string
         */
        private function getPeerType(string $type): string
        {
            if(in_array(strtolower($type), InternetPeerType::ALL))
                return PeerType::INTERNET;

            if(in_array(strtolower($type), UserPeerType::ALL))
                return PeerType::USER;

            return PeerType::UNKNOWN;
        }

        /**
         * Parses a raw federated address and returns a ParsedFederatedAddress object
         *
         * @param string $federated_address
         * @return ParsedFederatedAddress
         * @throws UnsupportedPeerType
         */
        private function parseAddress(string $federated_address): ParsedFederatedAddress
        {
            $parsed_address = new ParsedFederatedAddress($federated_address);

            if($this->getPeerType($parsed_address->getPeerType()) === PeerType::UNKNOWN)
            {
                throw new UnsupportedPeerType($parsed_address->getPeerType());
            }

            return $parsed_address;
        }

        /**
         * Registers a new peer into the database
         *
         * @param string|Client $client_uuid
         * @param string $federated_address
         * @return void
         * @throws DatabaseException
         * @throws UnsupportedPeerType
         */
        public function registerPeer(string|Client $client_uuid, string $federated_address): void
        {
            // If the client_uuid is a Client object, get the UUID from it
            if ($client_uuid instanceof Client)
            {
                $client_uuid = $client_uuid->getUuid();
            }

            // Check if the peer type is supported by the server
            $parsed_address = $this->parseAddress($federated_address);

            try
            {
                // Generate a query to insert the peer into the database
                $query_builder = Database::getConnection()->createQueryBuilder();
                $query_builder->insert(DatabaseTables::PEERS);
                $timestamp = time();

                $query_builder->values([
                    'federated_address' => $query_builder->createNamedParameter($parsed_address->getAddress()),
                    'client_first_seen' => $query_builder->createNamedParameter($client_uuid),
                    'client_last_seen' => $query_builder->createNamedParameter($client_uuid),
                    'active_restriction' => $query_builder->createNamedParameter(null, ParameterType::NULL),
                    'discovered_timestamp' => $query_builder->createNamedParameter($timestamp, ParameterType::INTEGER),
                    'seen_timestamp' => $query_builder->createNamedParameter($timestamp, ParameterType::INTEGER),
                ]);

                $query_builder->executeStatement();
            }
            catch(Exception $e)
            {
                throw new DatabaseException(sprintf('Failed to register peer %s: %s', $parsed_address->getAddress(), $e->getMessage()), $e);
            }

            Log::info(Misc::FEDERATIONLIB, sprintf('Registered new peer: %s', $parsed_address->getAddress()));
        }

        public function cachePeerObject(Peer $peer): void
        {
            if(!Configuration::isRedisEnabled() && !Configuration::isPeerObjectsCached())
            {
                return;
            }


        }

        /**
         * Fetches a peer from the database by its federated address
         *
         * @param string $federated_address
         * @return Peer
         * @throws DatabaseException
         * @throws PeerNotFoundException
         * @throws UnsupportedPeerType
         */
        public function getPeer(string $federated_address): Peer
        {
            // Check if the peer type is supported by the server
            $parsed_address = $this->parseAddress($federated_address);

            try
            {
                $query_builder = Database::getConnection()->createQueryBuilder();
                $query_builder->select('*');
                $query_builder->from(DatabaseTables::PEERS);
                $query_builder->where('federated_address = :federated_address');
                $query_builder->setParameter('federated_address', $parsed_address->getAddress());
                $query_builder->setMaxResults(1);

                $result = $query_builder->executeQuery();

                if($result->rowCount() === 0)
                {
                    throw new PeerNotFoundException($parsed_address->getAddress());
                }

                return Peer::fromArray($result->fetchAssociative());
            }
            catch(PeerNotFoundException $e)
            {
                throw $e;
            }
            catch(Exception $e)
            {
                throw new DatabaseException(sprintf('Failed to get peer %s: %s', $parsed_address->getAddress(), $e->getMessage()), $e);
            }
        }
    }