<?php

    /** @noinspection PhpMissingFieldTypeInspection */

    namespace FederationLib\Objects;

    use FederationCLI\Utilities;
    use FederationLib\Classes\Security;
    use FederationLib\Interfaces\SerializableObjectInterface;

    class Client implements SerializableObjectInterface
    {
        /**
         * @var string
         */
        private $uuid;

        /**
         * @var bool
         */
        private $enabled;

        /**
         * @var string|null
         */
        private $name;

        /**
         * @var string|null
         */
        private $description;

        /**
         * @var string|null
         */
        private $secret_totp;

        /**
         * @var int
         */
        private $query_permission;

        /**
         * @var int
         */
        private $update_permission;

        /**
         * @var string[]
         */
        private $flags;

        /**
         * @var int
         */
        private $created_timestamp;

        /**
         * @var int
         */
        private $updated_timestamp;

        /**
         * @var int
         */
        private $seen_timestamp;

        /**
         * Client constructor.
         */
        public function __construct()
        {
            $this->enabled = true;
            $this->flags = [];
            $this->query_permission = 1;
            $this->update_permission = 0;
        }

        /**
         * Returns the client's unique ID.
         *
         * @return string
         */
        public function getUuid(): string
        {
            return $this->uuid;
        }

        /**
         * @return bool
         */
        public function isEnabled(): bool
        {
            return $this->enabled;
        }

        /**
         * @param bool $enabled
         */
        public function setEnabled(bool $enabled): void
        {
            $this->enabled = $enabled;
        }

        /**
         * @return string|null
         */
        public function getName(): string
        {
            return $this->name;
        }

        /**
         * @param string|null $name
         */
        public function setName(?string $name): void
        {
            $this->name = $name;
        }

        /**
         * @return string|null
         */
        public function getDescription(): ?string
        {
            return $this->description;
        }

        /**
         * @param string|null $description
         */
        public function setDescription(?string $description): void
        {
            $this->description = $description;
        }

        /**
         * Enables authentication for the client, returns the secret.
         *
         * @return string
         */
        public function enableAuthentication(): string
        {
            $this->secret_totp = Security::generateSecret();
            return $this->secret_totp;
        }

        /**
         * Disables authentication for the client, wipes the secret.
         *
         * @return void
         */
        public function disableAuthentication(): void
        {
            $this->secret_totp = null;
        }

        /**
         * Validates the given authentication code against the client's secret.
         *
         * @param string|null $authentication_code
         * @param int|null $timestamp
         * @return bool
         */
        public function validateAuthentication(?string $authentication_code=null, ?int $timestamp=null): bool
        {
            if($this->secret_totp === null)
            {
                // Always authenticate if authentication is disabled.
                return true;
            }

            if($authentication_code === null)
            {
                // Authentication code not provided but required.
                return false;
            }

            // Authentication Code: sha1(client_id + totp_code)
            return hash('sha1', $this->uuid . Security::generateCode($this->secret_totp, $timestamp)) === $authentication_code;
        }

        /**
         * Returns the client's query permission level.
         *
         * @return int
         */
        public function getQueryPermission(): int
        {
            return $this->query_permission;
        }

        /**
         * Sets the client's query permission level.
         *
         * @param int $query_permission
         */
        public function setQueryPermission(int $query_permission): void
        {
            $this->query_permission = $query_permission;
        }

        /**
         * Returns the client's update permission level.
         *
         * @return int
         */
        public function getUpdatePermission(): int
        {
            return $this->update_permission;
        }

        /**
         * Sets the client's update permission.
         *
         * @param int $update_permission
         */
        public function setUpdatePermission(int $update_permission): void
        {
            $this->update_permission = $update_permission;
        }

        /**
         * Returns the client's flags.
         *
         * @return string[]
         */
        public function getFlags(): array
        {
            return $this->flags;
        }

        /**
         * Sets an array of flags for the client.
         * This function overrides any existing flags.
         *
         * @param string[] $flags
         */
        public function setFlags(array $flags): void
        {
            $this->flags = $flags;
        }

        /**
         * Appends a flag to the client's flags.
         *
         * @param string $flag
         * @return void
         */
        public function appendFlag(string $flag): void
        {
            if(!in_array($flag, $this->flags))
            {
                $this->flags[] = $flag;
            }
        }

        /**
         * Removes a flag from the client's flags.
         *
         * @param string $flag
         * @return void
         */
        public function removeFlag(string $flag): void
        {
            $this->flags = array_diff($this->flags, [$flag]);
        }

        /**
         * Returns True if the client has the given flag.
         *
         * @param string $flag
         * @return bool
         */
        public function hasFlag(string $flag): bool
        {
            return in_array($flag, $this->flags);
        }

        /**
         * Returns the Unix Timestamp for when the client was last created on the server
         *
         * @return int
         */
        public function getCreatedTimestamp(): int
        {
            return $this->created_timestamp;
        }

        /**
         * Returns the Unix Timestamp for when the client was last updated on the server.
         *
         * @return int
         */
        public function getUpdatedTimestamp(): int
        {
            return $this->updated_timestamp;
        }

        /**
         * Returns the Unix Timestamp for when the client was last seen by the server.
         *
         * @return int
         */
        public function getSeenTimestamp(): int
        {
            return $this->seen_timestamp;
        }

        /**
         * Returns an array representation of the object
         *
         * @return array
         */
        public function toArray(): array
        {
            $flags = null;

            if($this->flags !== null && count($this->flags) > 0)
            {
                $flags = implode(',', $this->flags);
            }

            return [
                'uuid' => $this->uuid,
                'enabled' => $this->enabled,
                'name' => $this->name,
                'description' => $this->description,
                'secret_totp' => $this->secret_totp,
                'query_permission' => $this->query_permission,
                'update_permission' => $this->update_permission,
                'flags' => $flags,
                'created_timestamp' => $this->created_timestamp,
                'updated_timestamp' => $this->updated_timestamp,
                'seen_timestamp' => $this->seen_timestamp,
            ];
        }

        /**
         * Constructs object from an array representation.
         *
         * @param array $array
         * @return Client
         */
        public static function fromArray(array $array): Client
        {
            $client = new self();

            $client->uuid = $array['uuid'] ?? null;
            $client->enabled = Utilities::parseBoolean($array['enabled']);
            $client->name = $array['name'] ?? null;
            $client->description = $array['description'] ?? null;
            $client->secret_totp = $array['secret_totp'] ?? null;
            $client->query_permission = $array['query_permission'] ?? 0;
            $client->update_permission = $array['update_permission'] ?? 0;
            if(isset($array['flags']))
            {
                $client->flags = explode(',', $array['flags']);
            }
            else
            {
                $client->flags = [];
            }
            $client->created_timestamp = $array['created_timestamp'] ?? 0;
            $client->updated_timestamp = $array['updated_timestamp'] ?? 0;
            $client->seen_timestamp = $array['seen_timestamp'] ?? 0;

            return $client;
        }

        /**
         * Returns the object identifier.
         *
         * @return string
         */
        public function __toString(): string
        {
            return sprintf('Client<%s>', $this->uuid ?? spl_object_hash($this));
        }
    }